#include "albumart.h"

int initModuleEx(HWND hwndParent, HINSTANCE hDllInstance, LPCSTR szPath)
{
	g_hParent = hwndParent;
	g_hInstance = hDllInstance;
	StringCchCopy(g_szPath, sizeof(g_szPath), szPath);

	if (!CreateMessageHandler())
		return 1;

	// Get RC Settings
	g_Frequency = GetRCInt("AlbumArtUpdateFrequency", 100);
	GetRCLine("AlbumArtOnChange", g_szOnChange, sizeof(g_szOnChange), "");
	GetRCLine("AlbumArtParsePath", g_szParsedPath, sizeof(g_szParsedPath), "");
	PathUnquoteSpaces(g_szParsedPath);
	GetRCLine("AlbumArtNoArtPath", g_szNoArtPath, sizeof(g_szNoArtPath), "");
	PathUnquoteSpaces(g_szNoArtPath);

	// Start the timer
	SetTimer(g_hwndMessageHandler, 1337, g_Frequency, NULL);

	// Initialize Variables
	g_iCurrentTrack = -2;
	UpdateInformation();

	return 0;
}

void quitModule(HINSTANCE hDllInstance)
{
	KillTimer(g_hwndMessageHandler, 1337);

	if (g_hwndMessageHandler)
	{
		SendMessage(GetLitestepWnd(),LM_UNREGISTERMESSAGE,(WPARAM)g_hwndMessageHandler, (LPARAM)g_lsMessages);
		DestroyWindow(g_hwndMessageHandler);
	}

	g_hwndMessageHandler = NULL;

	if (!UnregisterClass(g_szMsgHandler, hDllInstance))
		MessageBox(NULL, "UnregisterClass Failed!", "AlbumArt Error", MB_OK);
}

LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case LM_REFRESH:
		{
			return 0;
		}
	case LM_GETREVID:
		{
			StringCchPrintf((char*)lParam, 64, "%s: %s", g_szAppName, g_rcsRevision);
			size_t uLength;
			if (SUCCEEDED(StringCchLength((char*)lParam, 64, &uLength)))
				return uLength;
			lParam = NULL;
			return 0;
		}
	case WM_TIMER:
		{
			switch (wParam)
			{
			case 1337:
				{
					UpdateInformation();
				}
			}
			return 0;
		}
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

void UpdateInformation()
{
	HANDLE hWinamp;
	ULONG dWinamp;

	// Get Winamps HWND
	HWND hwndWA2 = FindWindow("Winamp v1.x", NULL);
	if (hwndWA2 == NULL)
		return; // Winamp isn't running

	// Make sure the track has changed
	int Track = SendMessage(hwndWA2, WM_USER, 0, 125);
	if (Track == g_iCurrentTrack)
		return; // The track hasn't changed
	g_iCurrentTrack = Track;

	// Open a handle to Winamp
	GetWindowThreadProcessId(hwndWA2, &dWinamp);
	hWinamp = OpenProcess(PROCESS_ALL_ACCESS, false, dWinamp);
	if (hWinamp == NULL)
		return; // Failed to open Winamp :/

	// Get the path to the currently playing file
	char szTrackPath[MAX_PATH];
	void* winampTrack = (void*)SendMessage(hwndWA2, WM_USER, Track, 211);
	ReadProcessMemory(hWinamp, winampTrack, &szTrackPath, sizeof(szTrackPath), NULL);
	CloseHandle(hWinamp);

	if (szTrackPath[4] == ':') // foobar2000...
		StringCchCopy(szTrackPath, sizeof(szTrackPath), (LPCSTR)(szTrackPath + 7));

	if (strcmp(g_szParsedPath, "") != 0)
	{
		// Try to get the cover from the ID3 tag
		ID3_Tag myTag(szTrackPath);
		ID3_Frame* myFrame = myTag.Find(ID3FID_PICTURE, ID3FN_PICTURETYPE, ID3PT_COVERFRONT);

		if (myFrame != NULL)
		{
			myFrame->Field(ID3FN_DATA).ToFile(g_szParsedPath);
			SetEvar("AlbumArtPath", g_szParsedPath);
			StringCchCopy(g_szArtPath, sizeof(g_szArtPath), g_szParsedPath);

			//TODO::Run a CRC32 check to make sure the file actually changed
			LSExecute(g_hParent, g_szOnChange, SW_SHOWNORMAL);
			return;
		}
	}

	// We didn't find any ID3 cover, so lets check for folder.jpg
	char szArtPath[MAX_PATH] = {0};
	StringCchCopy(szArtPath, sizeof(szArtPath), szTrackPath);
	for (int i = sizeof(szArtPath); i > 0; i--)
	{
		if (szArtPath[i] == '\\')
		{
			szArtPath[++i] = 0;
			break;
		}
	}
	StringCchCat(szArtPath, sizeof(szArtPath), "folder.jpg");

	if (File_Exists(szArtPath))
	{
		SetEvar("AlbumArtPath", szArtPath);
	}
	else
	{
		SetEvar("AlbumArtPath", g_szNoArtPath);
	}
	if (strcmp(g_szArtPath, szArtPath))
	{
		StringCchCopy(g_szArtPath, sizeof(g_szArtPath), szArtPath);
		LSExecute(g_hParent, g_szOnChange, SW_SHOWNORMAL);
	}
}

void SetEvar(LPCSTR szEvar, LPCSTR szValue)
{
	char szNewValue[MAX_LINE_LENGTH];
	StringCchCopy(szNewValue, sizeof(szNewValue), szValue);
	PathQuoteSpaces(szNewValue);
	LSSetVariable(szEvar, szNewValue);
}

bool CreateMessageHandler()
{
	WNDCLASSEX wc = {0};
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.lpfnWndProc = MessageHandlerProc;
	wc.hInstance = g_hInstance;
	wc.lpszClassName = g_szMsgHandler;
	wc.hIconSm = 0;
	wc.style = CS_NOCLOSE;

	if (!RegisterClassEx(&wc))
	{
		MessageBox(NULL, "RegisterClassEx Failed!", "AlbumArt Error", MB_OK);
		return false;
	}

	g_hwndMessageHandler = CreateWindowEx(WS_EX_TOOLWINDOW, g_szMsgHandler, 0, WS_POPUP, 0, 0, 0, 0, g_hParent, 0, g_hInstance, 0);

	if (!g_hwndMessageHandler)
	{
		MessageBox(NULL, "CreateWindowEx Failed!", "AlbumArt Error", MB_OK);
		return false;
	}
	
	SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM)g_hwndMessageHandler, (LPARAM)g_lsMessages);

	return true;
}

BOOL APIENTRY DllMain(HINSTANCE hInst, DWORD dwReason, LPVOID /* pvReserved */)
{
	if (dwReason == DLL_PROCESS_ATTACH)
		DisableThreadLibraryCalls(hInst);

	return TRUE;
}

bool File_Exists (char* szFilePath)
{
	return GetFileAttributes(szFilePath) != INVALID_FILE_ATTRIBUTES;
}